#include "object3D.h"

Object3D::Object3D() {}

/**
 * Creates a new Object3D with given vertices, model and a VAO handle.
 */
Object3D::Object3D(std::vector<glm::vec3> in_vertices, glm::mat4 in_model, GLuint in_vaoHandle) {
    vertices = in_vertices;
    model = in_model;
    vaoHandle = in_vaoHandle;

    printf("Created an object with %d vertices.\n", vertices.size());

    worldVertices = std::vector<glm::vec3>();
    updateWorldVertices();
};

/**
 * Another constructor that also assignes a texture handle.
 */
Object3D::Object3D(std::vector<glm::vec3> in_vertices, glm::mat4 in_model, GLuint in_vaoHandle, GLuint in_textureHandle) {
    textureHandle = in_textureHandle;
    Object3D(in_vertices, in_model, in_vaoHandle);
};

/**
 * Updates the world space vertices using the matrix returned from getModel().
 */
void Object3D::updateWorldVertices() {
    worldVertices.clear();
    for (auto vertex : vertices) {
        worldVertices.push_back(glm::vec3(getModel() * glm::vec4(vertex, 1.0)));
    }
}

/**
 * Sets one vertex.
 */
void Object3D::setVertex(glm::vec3 vertex, int i) {
    vertices.at(i) = vertex;
}

/**
 * Updates the object.
 */
void Object3D::update(float dt) {
    //Do nothing
}

/**
 * Getter for the model matrix.
 */
glm::mat4 Object3D::getModel() {

    return this->model;
}

/**
 * Getter for the VAO.
 */
GLuint Object3D::getVAO() {

    return this->vaoHandle;
}

/**
 * Get the vertex count.
 */
unsigned int Object3D::getVertexCount() {

    return this->vertices.size();
}

/**
 * Getter for the world vertices.
 */
std::vector<glm::vec3> Object3D::getWorldVertices() {

    return this->worldVertices;
}
