#ifndef OBJECT3D_H
#define OBJECT3D_H
#include <stdlib.h>         // C++ standard library
#include <stdio.h>
#include <stdexcept>
#include <vector>            // We use the standard C++ stack implementation to create model matrix stacks
#include <GLEW/glew.h>      // OpenGL Extension Wrangler -
#include <GLFW/glfw3.h>     // Windows and input
#include <glm/glm.hpp>      // OpenGL math library
#include <glm/gtc/constants.hpp>

/**
 * Class to hold one 3D object: its vertices, VAO handle, model matrix. Optionally also the texture handle.
 */
class Object3D {
    protected:
        GLuint vaoHandle;
        GLuint textureHandle;
        std::vector<glm::vec3> vertices;
        std::vector<glm::vec3> worldVertices;
        glm::mat4 model;
    public:
        Object3D();
        Object3D(std::vector<glm::vec3> vertices, glm::mat4 model, GLuint vaoHandle);
        Object3D(std::vector<glm::vec3> vertices, glm::mat4 model, GLuint vaoHandle, GLuint textureHandle);
        virtual void update(float dt);
        virtual glm::mat4 getModel();
        GLuint getVAO();
        unsigned int getVertexCount();
        std::vector<glm::vec3> getWorldVertices();
        void setVertex(glm::vec3 vertex, int i);
        void updateWorldVertices();
};
#endif
