#ifndef RAYS_H
#define RAYS_H

#include <memory>
#include <algorithm> //Sorting
#include <glm/gtx/rotate_vector.hpp>
#include "shader_util.h"
#include "Object3D.h"
#include "ObjectCollection.h"

/**
 * Struct to hold one ray.
 * Ray has: start, direction, hit point,
 * object (it shoots to), vertex (of the object it shoots to), shift (of the ray)
 */
struct Ray {
    glm::vec3 start;
    glm::vec3 dir;
    glm::vec3 hit;
    Object3D* object;
    int vertexIndex;
    float shift;
    Ray(glm::vec3 in_start, glm::vec3 in_dir, Object3D* in_object, int in_vertexIndex, float in_shift) {
        start = in_start;
        dir = in_dir;
        object = in_object;
        vertexIndex = in_vertexIndex;
        shift = in_shift;
    }

    bool operator< (Ray const &r) const {
        /**
         * --Task--
         * We want to compare different rays in order to sort them in a counter-clockwise order.
         */

        return true;
    }
};

/**
 * Class that holds all the rays and traces them against the testableObjects upon update.
 * It also constructs and holds the shadow polygon.
 * Draws the rays and shadow polygon.
 */
class Rays
{
    public:
        Rays();
        void init(const ObjectCollection *testableObjects, shader_prog *shader, shader_prog *polgyonShader);
        void update(const ObjectCollection *testableObjects);
        void draw();
        void mouseMove(double x, double y, double width, double height);

        GLuint raysVAO;
        GLuint raysVBO; //We store the VBO handles in order to change the data in them
        GLuint shadowPolygonVBO;

    protected:
        std::vector<Ray> rays;
        ObjectCollection testableObjects;
        shader_prog* lineShader;
        shader_prog* polygonShader;
        Object3D* shadowPolygon;

        void initRays(const ObjectCollection *testableObjects);
        void initShadowPolygon();
        void updateShadowPolygon();
        void drawShadowPolygon();

        glm::vec3 rayTrace(Ray &ray, const ObjectCollection *testableObjects);
        float rayToLineSegment(const Ray &ray, const glm::vec3 &lineStart, const glm::vec3 &lineEnd);
    private:
};

#endif // RAYS_H
