#ifndef SCENE_H
#define SCENE_H

#include <vector>
#include <random>
#include <memory>
#include "Object3D.h"
#include "RotatingObject3D.h"
#include "shader_util.h"
#include "texture_util.h"
#include "ObjectCollection.h"


/**
 * This class manages all the objects in our scene.
 * Has the functions to create, update and draw them.
 */
class Scene
{
    public:
        Scene();
        void init(shader_prog* in_lineShader, shader_prog* in_floorShader, float width);
        Object3D createBox(float width);
        Object3D createLetterC();
        Object3D createLetterG();
        void update(double dt);
        void draw();
        void createTriangles();
        RotatingObject3D* createTriangle(glm::vec2 position);
        ObjectCollection* getTestableObjects();
        GLuint createFloor(float width);
    protected:
        ObjectCollection testableObjects;
        shader_prog* lineShader;
        shader_prog* floorShader;
        GLuint floorVAO;
        GLuint floorTextureHandle;
        std::mt19937 generator;
        std::uniform_real_distribution<> distribution;

        void drawLineObject(Object3D& object);
        void drawFloor();
    private:
};

#endif // SCENE_H
